import * as React from "react";
import {observer} from "mobx-react-lite";
import {ButtonGroup, TableCell, TableRow} from "@material-ui/core";
import {useReadonlyContext} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {ComponentPermissionButton} from "./ComponentPermissionButton";
import {Permission} from "../../stores/RoleStore";
import {GrantedButton} from "./GrantedButton";
import {DeniedButton} from "./DeniedButton";
import {ComponentPermissionGroup} from "../../models/ComponentPermissionGroup";

export type ComponentPermissionRowProps = {
    component: ComponentPermissionGroup
}

export const ComponentPermissionRow: React.FunctionComponent<ComponentPermissionRowProps> = observer((props) => {
    const {component} = props;
    const readOnly = useReadonlyContext();
    const setAllForComponent = function (granted: boolean) {
        component.actions.forEach(action => action.rolePermission.viewModel.permitted = granted);
    };

    const actions = component.actions.sort((a, b) => {
        const aRank = ['create', 'read', 'update', 'delete', 'design', 'run', 'apply'].indexOf(a.componentPermission.action_id);
        const bRank = ['create', 'read', 'update', 'delete', 'design', 'run', 'apply'].indexOf(b.componentPermission.action_id);
        if (aRank >= 0 && bRank >= 0) {
            return aRank - bRank
        } else if (aRank >= 0) {
            return -1;
        } else if (bRank >= 0) {
            return 1;
        } else {
            return 0;
        }
    }).reduce((buckets: Permission[][], action, i) => {
        if ((i % 6) === 0) {
            buckets.push([]);
        }
        buckets[buckets.length - 1].push(action);
        return buckets;
    }, []);

    return (
        <TableRow key={component.target.target_id}>
            <TableCell component="th" scope="row">
                {component.target.target_display_name}
            </TableCell>
            <TableCell align="left">
                {
                    actions.map(row => (
                        <React.Fragment key={row[0].componentPermission.id as string}>
                            <ButtonGroup color="primary" aria-label="outlined primary button group">
                                {
                                    row.map((action: Permission) => {
                                        return <ComponentPermissionButton key={action.componentPermission.id as string}
                                                                          disabled={readOnly}
                                                                          componentAction={action}/>
                                    })
                                }
                            </ButtonGroup>
                            <br/>
                        </React.Fragment>
                    ))
                }
            </TableCell>
            {!readOnly && <TableCell align="right">
                <ButtonGroup color="primary" aria-label="outlined primary button group">
                    <GrantedButton
                        onClick={() => setAllForComponent(true)}
                    >
                        {t`Grant All`}
                    </GrantedButton>
                    <DeniedButton
                        onClick={() => setAllForComponent(false)}
                    >
                        {t`Revoke All`}
                    </DeniedButton>
                </ButtonGroup>
            </TableCell>}
        </TableRow>
    );
});
